import 'dart:ffi';

import 'package:anagog_flutter_platform_interface/anagog_flutter_platform_interface.dart';
import 'package:anagog_flutter_platform_interface/src/model/micromoment.dart';
import 'package:flutter/services.dart';

class AnagogFlutterMethodChannel extends AnagogFlutterPlatform {
  final _methodChannel =
      const MethodChannel('com.anagog.flutter/method_channel');
  final _eventChannel =
      const EventChannel('com.anagog.flutter/method_channel/events');

  ///Simulate a campaign, use this API to test your campaigns as defined.
  ///[campaignIdentifier] A campaign identifier to simulate. Pass null to simulates all your campaigns
  @override
  Future simulateCampaign(String? campaignIdentifier) async =>
      _methodChannel.invokeMethod<Void>('simulateCampaign',
          <String, dynamic>{'campaign_identifier': campaignIdentifier});

  ///Sync campaigns
  ///Download all your campaigns to your app
  @override
  Future syncCampaigns() async =>
      _methodChannel.invokeMethod<Void>('syncCampaigns');

  ///Get the app custom micro segments map
  @override
  Future<Map<dynamic, dynamic>?> getUserDefinedStat() async =>
      _methodChannel.invokeMapMethod("getUserDefinedStat");

  ///Set String custom micro segment value
  ///[arguments] Micro Segment values
  @override
  Future<dynamic> setUserDefinedString(Map<String, dynamic> arguments) async =>
      _methodChannel.invokeMethod("setUserDefinedString", arguments);

  ///Get String custom micro segment
  ///[name] Stat name
  @override
  Future<String?> getUserDefinedString(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedString", name);

  ///Set Integer custom micro segment value
  ///[arguments] Micro Segment values
  @override
  Future<dynamic> setUserDefinedInteger(Map<String, dynamic> arguments) async =>
      _methodChannel.invokeMethod("setUserDefinedInteger", arguments);

  ///Get Integer custom micro segment
  ///[name] Stat name
  @override
  Future<int?> getUserDefinedInteger(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedInteger", name);

  ///Set Decimal custom micro segment value
  ///[arguments] Micro Segment values
  @override
  Future<dynamic> setUserDefinedDecimal(Map<String, dynamic> arguments) async =>
      _methodChannel.invokeMethod("setUserDefinedDecimal", arguments);

  ///Get Decimal custom micro segment
  ///[name] Stat name
  @override
  Future<double?> getUserDefinedDecimal(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedDecimal", name);

  ///Use to create report a [Micromoment] object from your app
  @override
  Future fireMicromoments(Micromoment micromoment) async =>
      _methodChannel.invokeMethod("fireMicromoments", micromoment.toJson());

  ///Create and share SDK report.
  ///Use this in case you want to share with Anagog SDK operation and debugging report for analyze
  @override
  Future share() async => _methodChannel.invokeMethod("share");

  ///Create an EventStream from SDK to your Flutter app.
  ///Use this API to listen for SDK events such as onCampaignTriggered, onNotificationClick
  @override
  Stream obtainEventStream() => _eventChannel.receiveBroadcastStream();

  ///Get SDK Version
  @override
  Future getVersion() async => _methodChannel.invokeMethod("getVersion");

  ///Request to generate Microsegments Snapshot report.
  @override
  Future requestMicrosegmentsSnapshotReport() async =>
      _methodChannel.invokeMethod("requestMicrosegmentsSnapshotReport");
}
