import 'package:anagog_flutter_platform_interface/src/model/location.dart';

///Anagog SDK MicroMoment object, represents campaign Micromoment event which app can pass to SDK
///[identifier] Micromoment identifier such as: 'Registration'
///[textParameters] A Map of Micromoment textParameters such as: {'Status': 'Completed'}
///[numericParameters] A Map of Micromoment numeric values such as: {'Score': 150}
///[locationParameters] A Map of Micromoment [Location] values such as: {'Home': Location(latitude: 32.5, longitude: 34.6)}
class Micromoment {
  final String identifier;
  final Map<String, String> textParameters;
  final Map<String, double> numericParameters;
  final Map<String, Location> locationParameters;

  Micromoment(
      {required this.identifier,
      required this.textParameters,
      required this.numericParameters,
      required this.locationParameters});

  Map<String, dynamic> toJson() => {
        'identifier': identifier,
        'textParameters': textParameters,
        'numericParameters': numericParameters,
        'locationParameters':
            locationParameters.map((k, e) => MapEntry(k, e.toJson()))
      };

  factory Micromoment.fromJson(Map<String, dynamic> json) {
    return Micromoment(
        identifier: json['identifier'] as String,
        textParameters: Map<String, String>.from(json['textParameters'] as Map),
        numericParameters:
            (json['numericParameters'] as Map<String, dynamic>).map(
          (k, e) => MapEntry(k, (e as num).toDouble()),
        ),
        locationParameters:
            (json['locationParameters'] as Map<String, dynamic>).map(
          (k, e) => MapEntry(k, Location.fromJson(e as Map<String, dynamic>)),
        ));
  }
}
